{ pkgs, ... }:


let
  customPlugins = {
      vim-zettel = pkgs.vimUtils.buildVimPlugin {
        name = "vim-zettel";
        src = pkgs.fetchFromGitHub {
          owner = "michal-h21";
          repo = "vim-zettel";
          rev = "929d90eec62e6f693c2702d2b6f76a93f2f1689d";
          sha256 = "1fzKmknfVlEYwqeXgbKITbb2/PJ023iJyMz6vak3qh4=";
        };
      };
      vimwiki-dev = pkgs.vimUtils.buildVimPlugin {
        name = "vimwiki-dev";
        src = pkgs.fetchFromGitHub {
          owner = "vimwiki";
          repo = "vimwiki";
          rev = "d035a3428c6343b4f218e51340ddf690ad8c57ee";
          sha256 = "000000000000000000000000PJ023iJyMz6vak3qh4=";
        };
      };
      taskwiki-t = pkgs.vimUtils.buildVimPluginFrom2Nix {
        pname = "taskwiki";
        version = "2021-09-22";
        src = pkgs.fetchFromGitHub {
          owner = "tools-life";
          repo = "taskwiki";
          rev = "c4392c36c01cfffcb06f460d722990b98bc53bdb";
          sha256 = "QrxFqu/Y6FykXGqIlK52+nDwmnBhubURl6v2Zc/KwPw=";
        };
        meta.homepage = "https://github.com/tools-life/taskwiki/";
      };
  };
  my-python-packages = python-packages: with python-packages; [
    tasklib
    pynvim
    six
    # other python packages you want
  ]; 
  python-with-my-packages = pkgs.python3.withPackages my-python-packages;
in
  { 
    environment.variables.EDITOR = "nvim";
    environment.shellAliases = {
      k = "vim -c :VimwikiIndex";
      kz = "vim -c :VimwikiIndex -c :ZettelNew";
    };
    environment.systemPackages = with pkgs // customPlugins; [ 
      nodejs
      python-with-my-packages
      #python3Packages.pynvim
      #python3Packages.six
      #python3Packages.tasklib
      #python3
  ( neovim.override {
  vimAlias = true;
  withPython3 = true;
  extraPython3Packages = pkgs: with pkgs; [ tasklib six ];
  configure = {
		packages.myPlugins = with pkgs.vimPlugins; {
			start = [ 
				vim-nix
				fzf-vim
				vim-elixir
				emmet-vim
				nerdtree
				goyo-vim
				vim-sneak
				lightline-vim
				csv-vim
				vimwiki
				coc-nvim
				coc-json
				coc-python
				coc-pairs
				coc-emmet
				LanguageClient-neovim
				ayu-vim
				rainbow_parentheses-vim
				vim-surround
				vim-fugitive
				#nvim-colorizer
				vim-devicons
				# testing
				#vim-processing
				#vim-tidal
				taskwiki
				vim-zettel
				#vimwiki-sync
        telescope-nvim
        # Clojure stuff
        conjure
        vim-sexp
        vim-sexp-mappings-for-regular-people
			];
		};
    customRC = ''
	set tabstop=2
	set showmatch
	set shiftwidth=2
	set noexpandtab
	set autoindent

	set foldmethod=indent   " fold based on indent
	set number
	set nocompatible
	set relativenumber
	set showtabline=2
	set autowriteall
			syntax on
	set list lcs=tab:\|\ 
	filetype plugin on

	set termguicolors
	let ayucolor="dark"
	colorscheme ayu

	" window
	nmap <leader>wh  :topleft  vnew<CR>
	nmap <leader>wl :botright vnew<CR>
	nmap <leader>wk    :topleft  new<CR>
	nmap <leader>wj  :botright new<CR>

			let mapleader = "\<Space>"

			" lightline colorscheme
			let g:lightline = {
						\ 'colorscheme': 'seoul256',
						\ }

			" moving splits with vim movements.
			:tnoremap <A-h> <C-\><C-N><C-w>h
			:tnoremap <A-j> <C-\><C-N><C-w>j
			:tnoremap <A-k> <C-\><C-N><C-w>k
			:tnoremap <A-l> <C-\><C-N><C-w>l
			:inoremap <A-h> <C-\><C-N><C-w>h
			:inoremap <A-j> <C-\><C-N><C-w>j
			:inoremap <A-k> <C-\><C-N><C-w>k
			:inoremap <A-l> <C-\><C-N><C-w>l
			:nnoremap <A-h> <leader>h
			:nnoremap <A-j> <leader>j
			:nnoremap <A-k> <leader>k
			:nnoremap <A-l> <leader>l
			:nnoremap <F5> "=strftime("%Y-%m-%d %H:%M:")
			:inoremap <F5> <C-R>=strftime("%Y-%m-%d %H:%M:")

      " exit normal mode in terminal
      :tnoremap <C-n> <C-\><C-n>

			let g:syntastic_always_populate_loc_list = 1
			let g:syntastic_auto_loc_list = 1
			let g:syntastic_check_on_open = 1
			let g:syntastic_check_on_wq = 0

      " Sexp in Conjure (since it's fennel)
      " let g:sexp_filetypes = [ "fennel" ]

      let g:rainbow_active = 0
      autocmd BufReadPost,BufNewFile c,py,nix RainbowParenthesesToggleAll

			" keyboard commands
			map <leader>t :NERDTreeToggle<CR>
			map <leader>s :w<CR>
			map <leader>q :wq<CR>
      map <leader>r :RainbowParenthesesToggleAll<CR>
      map <leader>zs :ZettelSearch<CR>
      map <leader>zo :ZettelOpen<CR>
      map <leader>zn :ZettelNew<CR>


			" NERDTree 
			" open on directory
			autocmd StdinReadPre * let s:std_in=1

			" coc
			"
			" if hidden is not set, TextEdit might fail.
			set hidden

			" Some servers have issues with backup files, see #649
			set nobackup
			set nowritebackup

			" Better display for messages
			set cmdheight=2

			" You will have bad experience for diagnostic messages when it's default 4000.
			set updatetime=300

			" don't give |ins-completion-menu| messages.
			set shortmess+=c

			" always show signcolumns
			set signcolumn=yes

			" Use tab for trigger completion with characters ahead and navigate.
			" Use command ':verbose imap <tab>' to make sure tab is not mapped by other plugin.
			inoremap <silent><expr> <TAB>
						\ pumvisible() ? "\<C-n>" :
						\ <SID>check_back_space() ? "\<TAB>" :
						\ coc#refresh()
			inoremap <expr><S-TAB> pumvisible() ? "\<C-p>" : "\<C-h>"

			function! s:check_back_space() abort
				let col = col('.') - 1
				return !col || getline('.')[col - 1]  =~# '\s'
			endfunction

			" Use <c-space> to trigger completion.
			inoremap <silent><expr> <c-space> coc#refresh()

			" Use <cr> to confirm completion, `<C-g>u` means break undo chain at current position.
			" Coc only does snippet and additional edit on confirm.
			inoremap <expr> <cr> pumvisible() ? "\<C-y>" : "\<C-g>u\<CR>"
			" Or use `complete_info` if your vim support it, like:
			" inoremap <expr> <cr> complete_info()["selected"] != "-1" ? "\<C-y>" : "\<C-g>u\<CR>"

			" Use `[g` and `]g` to navigate diagnostics
			nmap <silent> [g <Plug>(coc-diagnostic-prev)
			nmap <silent> ]g <Plug>(coc-diagnostic-next)

			" Remap keys for gotos
			nmap <silent> gd <Plug>(coc-definition)
			nmap <silent> gy <Plug>(coc-type-definition)
			nmap <silent> gi <Plug>(coc-implementation)
			nmap <silent> gr <Plug>(coc-references)

			" Use K to show documentation in preview window
			nnoremap <silent> K :call <SID>show_documentation()<CR>

			function! s:show_documentation()
				if (index(['vim','help'], &filetype) >= 0)
					execute 'h '.expand('<cword>')
				else
					call CocAction('doHover')
				endif
			endfunction

			" Highlight symbol under cursor on CursorHold
			autocmd CursorHold * silent call CocActionAsync('highlight')

			" Remap for rename current word
			nmap <leader>rn <Plug>(coc-rename)

			" Remap for format selected region
			xmap <leader>f  <Plug>(coc-format-selected)
			nmap <leader>f  <Plug>(coc-format-selected)

			augroup mygroup
				autocmd!
				" Setup formatexpr specified filetype(s).
				autocmd FileType typescript,json setl formatexpr=CocAction('formatSelected')
				" Update signature help on jump placeholder
				autocmd User CocJumpPlaceholder call CocActionAsync('showSignatureHelp')
			augroup end

			" Remap for do codeAction of selected region, ex: `<leader>aap` for current paragraph
			xmap <leader>a  <Plug>(coc-codeaction-selected)
			nmap <leader>a  <Plug>(coc-codeaction-selected)

			" Remap for do codeAction of current line
			nmap <leader>ac  <Plug>(coc-codeaction)
			" Fix autofix problem of current line
			nmap <leader>qf  <Plug>(coc-fix-current)

			" Create mappings for function text object, requires document symbols feature of languageserver.
			xmap if <Plug>(coc-funcobj-i)
			xmap af <Plug>(coc-funcobj-a)
			omap if <Plug>(coc-funcobj-i)
			omap af <Plug>(coc-funcobj-a)

			" Use <C-d> for select selections ranges, needs server support, like: coc-tsserver, coc-python
			nmap <silent> <C-d> <Plug>(coc-range-select)
			xmap <silent> <C-d> <Plug>(coc-range-select)

			" Use `:Format` to format current buffer
			command! -nargs=0 Format :call CocAction('format')

			" Use `:Fold` to fold current buffer
			command! -nargs=? Fold :call     CocAction('fold', <f-args>)

			" use `:OR` for organize import of current buffer
			command! -nargs=0 OR   :call     CocAction('runCommand', 'editor.action.organizeImport')

			" Add status line support, for integration with other plugin, checkout `:h coc-status`
			" set statusline^=%{coc#status()}%{get(b:,'coc_current_function',"")}

			" Using CocList
			" Show all diagnostics
			nnoremap <silent> <space>a  :<C-u>CocList diagnostics<cr>
			" Manage extensions
			nnoremap <silent> <space>e  :<C-u>CocList extensions<cr>
			" Show commands
			nnoremap <silent> <space>c  :<C-u>CocList commands<cr>
			" Find symbol of current document
			nnoremap <silent> <space>o  :<C-u>CocList outline<cr>
			" Search workspace symbols
			nnoremap <silent> <space>s  :<C-u>CocList -I symbols<cr>
			" Do default action for next item.
			nnoremap <silent> <space>j  :<C-u>CocNext<CR>
			" Do default action for previous item.
			nnoremap <silent> <space>k  :<C-u>CocPrev<CR>
			" Resume latest coc list
			nnoremap <silent> <space>p  :<C-u>CocListResume<CR>

			au BufNewFile,BufRead *.frag,*.vert,*.fp,*.vp,*.glsl setf glsl
				'';
			};
		}
	)];
}
